package org.ssssssss.magicboot.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;
import org.ssssssss.magicboot.constant.URLConstant;
import org.ssssssss.magicboot.entity.Filter;
import org.ssssssss.magicboot.utils.CrmRequestUtil;

import java.util.*;

/**
 * 要货计划
 *
 * @param
 * @return null
 * @Author weiloong_zhang
 */
@RestController
@RequestMapping("/api/goods_plan")
@Slf4j
public class GoodsPlanController {

    private RestTemplate restTemplate = new RestTemplate();

    public static void main(String[] args) {
        //new GoodsPlanController().syncGoodsPlan();
        new GoodsPlanController().updateGoodsPlan();
    }

    /**
     * 同步新增要货计划
     *
     * @return void
     * @Author weiloong_zhang
     */
    @PostMapping("/sync/goodsPlan")
    public void syncGoodsPlan() {
        CrmRequestUtil crmRequestUtil = new CrmRequestUtil();
        log.info("开始同步要货计划");

        //要货计划请求
        JSONObject getGoodsPlan = crmRequestUtil.getCRMList(Arrays.asList(
                new Filter("EQ", "name", Arrays.asList("YHJH20250508-0002")),
                new Filter("EQ", "life_status", Arrays.asList("normal"))
        ), "purchase_plan__c");

        JSONObject goodsPlanRes = new JSONObject();

        //开始发起请求
        try {
            goodsPlanRes = restTemplate.postForObject(URLConstant.GET_CUSTOMIZE_LIST_URL, getGoodsPlan, JSONObject.class);
        } catch (RestClientException e) {
            e.printStackTrace();
        }

        //判断是否请求成功
        if (!"success".equals(goodsPlanRes.getString("errorDescription")) || goodsPlanRes.getJSONObject("data").getJSONArray("dataList").isEmpty()) {
            log.info("请求失败或数据不存在");
            return;
        }

        //请求成功
        for (Object goodsPlanObj : goodsPlanRes.getJSONObject("data").getJSONArray("dataList")) {
            //将Obj转为JSONObject
            JSONObject goodsPlanData = JSON.parseObject(JSON.toJSONString(goodsPlanObj));

            System.out.println("当前正在处理的要货计划数据为：" + goodsPlanData);

            //获取当前单据的唯一id
            String goodsPlanId = goodsPlanData.getString("_id");

            //获取当前单据名称
            String goodsPlanName = goodsPlanData.getString("name");

            //获取当前单据的创建时间
            String createTime = goodsPlanData.getString("create_time");

            //开始查询该单据是否已经集成过
            JSONObject logRes = new JSONObject();

            try {
                logRes = restTemplate.getForObject("http://localhost:18088/Log/query/log_data?table=send_log_goodsplan&log_type=GP&dataId=" + goodsPlanId + "&mark=" + createTime, JSONObject.class);
            } catch (RestClientException e) {
                e.printStackTrace();
            }

            if (!logRes.getJSONArray("data").isEmpty()) {
                log.info("当前单据已经集成过了");
                continue;
            }

            log.info("当前单据没有集成过");

            //开始封装传输至ERP的请求
            log.info("开始封装请求");

            Map erpGoodsPlanReq = new HashMap<>();
            erpGoodsPlanReq.put("f9w5_crmid", goodsPlanId);
            erpGoodsPlanReq.put("billno", goodsPlanName);//单据编号
            erpGoodsPlanReq.put("billtype_number", "f9w5_sm_purchaseplan_BT");//单据类型
            erpGoodsPlanReq.put("org_number", "zyierp");//组织编码

            //开始获取需求日期
            long biztime = goodsPlanData.getLong("requirement_date__c");

            Date date = new Date(biztime);
            erpGoodsPlanReq.put("biztime", date);//需求日期

            //部门编码
            String deptCode = "";

            List deptIdList = goodsPlanData.getJSONArray("data_own_department");

            //开始查询部门编码
            JSONObject deptReq = crmRequestUtil.getCRMList(Arrays.asList(
                    new Filter("EQ", "_id", deptIdList)
            ), "DepartmentObj");

            JSONObject deptRes = new JSONObject();

            try {
                deptRes = restTemplate.postForObject(URLConstant.GET_CRM_LIST_URL, deptReq, JSONObject.class);
            } catch (RestClientException e) {
                e.printStackTrace();
            }

            if ("success".equals(deptRes.getString("errorDescription")) && !deptRes.getJSONObject("data").getJSONArray("dataList").isEmpty()) {
                JSONObject deptData = deptRes.getJSONObject("data").getJSONArray("dataList").getJSONObject(0);
                deptCode = deptData.getString("field_Vj6sf__c") != null ? deptData.getString("field_Vj6sf__c") : "";
            }

            erpGoodsPlanReq.put("dept", "zyierp");//todo 归属部门

            //获取负责人编码
            String ownerCode = "";

            List ownerIdList = goodsPlanData.getJSONArray("owner");

            JSONObject ownerReq = crmRequestUtil.getCRMList(Arrays.asList(
                    new Filter("EQ", "_id", ownerIdList)
            ), "PersonnelObj");

            JSONObject ownerRes = new JSONObject();

            try {
                ownerRes = restTemplate.postForObject(URLConstant.GET_CRM_LIST_URL, ownerReq, JSONObject.class);
            } catch (RestClientException e) {
                e.printStackTrace();
            }

            if ("success".equals(ownerRes.getString("errorDescription")) && !ownerRes.getJSONObject("data").getJSONArray("dataList").isEmpty()) {
                JSONObject ownerData = ownerRes.getJSONObject("data").getJSONArray("dataList").getJSONObject(0);
                ownerCode = ownerData.getString("field_T1xid__c") != null ? ownerData.getString("field_T1xid__c") : "";
            }

            erpGoodsPlanReq.put("operator", "ID-000011");//todo 负责人

            //根据要货计划查询要货计划明细
            JSONObject getGoodsPlanDetail = crmRequestUtil.getCRMList(Arrays.asList(
                    new Filter("EQ", "purchase_plan__c", Arrays.asList(goodsPlanId))
            ), "purchase_plan_details__c");

            JSONObject goodsPlanDetailRes = new JSONObject();

            try {
                goodsPlanDetailRes = restTemplate.postForObject(URLConstant.GET_CUSTOMIZE_LIST_URL, getGoodsPlanDetail, JSONObject.class);
            } catch (RestClientException e) {
                e.printStackTrace();
            }

            System.out.println("要货计划明细查询结果为：" + goodsPlanDetailRes);

            if (!"success".equals(goodsPlanDetailRes.getString("errorDescription")) || goodsPlanDetailRes.getJSONObject("data").getJSONArray("dataList").isEmpty()) {
                log.info("要货计划明细查询失败不存在当前要货计划明细");
                continue;
            }

            log.info("要货计划明细存在，继续向下执行");

            //明细列表
            List goodsPlanDetailList = new ArrayList<>();

            for (Object goodsPlanDetailObj : goodsPlanDetailRes.getJSONObject("data").getJSONArray("dataList")) {
                JSONObject goodsPlanDetailData = JSON.parseObject(JSON.toJSONString(goodsPlanDetailObj));

                System.out.println("当前正在处理的要货计划明细数据为：" + goodsPlanDetailData);

                //开始封装要货计划明细数据
                Map erpGoodsPlanDetailReq = new HashMap<>();
                erpGoodsPlanDetailReq.put("f9w5_crmentryid", goodsPlanDetailData.getString("_id"));//crm明细id
                erpGoodsPlanDetailReq.put("material_number", goodsPlanDetailData.getString("material_id__c__r"));//物料编号
                erpGoodsPlanDetailReq.put("entryorg_number", "zyierp");//组织编码
                erpGoodsPlanDetailReq.put("qty", goodsPlanDetailData.getBigDecimal("quantity__c") != null ? goodsPlanDetailData.getBigDecimal("quantity__c") : 0);//数量
                //erpGoodsPlanDetailReq.put("lot_number", goodsPlanDetailData.getString("batch_number__c") != null ? goodsPlanDetailData.getString("batch_number__c") : "");//批号
                erpGoodsPlanDetailReq.put("entrycomment", goodsPlanDetailData.getString("remark__c") != null ? goodsPlanDetailData.getString("remark__c") : "");//备注
                erpGoodsPlanDetailReq.put("f9w5_contractno", goodsPlanDetailData.getString("sales_contract_number__c__r") != null ? goodsPlanDetailData.getString("sales_contract_number__c__r") : "");//销售合同编号

                //开始查询项目号
                String projectId = goodsPlanDetailData.getString("project_name__c") != null ? goodsPlanDetailData.getString("project_name__c") : "";

                //开始查询项目号
                JSONObject getProjectReq = crmRequestUtil.getCRMList(Arrays.asList(
                        new Filter("EQ", "_id", Arrays.asList(projectId))
                ), "NewOpportunityObj");

                JSONObject projectRes = new JSONObject();

                try {
                    projectRes = restTemplate.postForObject(URLConstant.GET_CUSTOMIZE_LIST_URL, getProjectReq, JSONObject.class);
                } catch (RestClientException e) {
                    e.printStackTrace();
                }

                String projectCode = "";

                if ("success".equals(projectRes.getString("errorDescription")) && !projectRes.getJSONObject("data").getJSONArray("dataList").isEmpty()) {
                    JSONObject projectData = projectRes.getJSONObject("data").getJSONArray("dataList").getJSONObject(0);
                    projectCode = projectData.getString("opportunity_id__c") != null ? projectData.getString("opportunity_id__c") : "";
                }

                erpGoodsPlanDetailReq.put("f9w5_project_number", projectCode);//项目号

                goodsPlanDetailList.add(erpGoodsPlanDetailReq);

            }

            erpGoodsPlanReq.put("billentry", goodsPlanDetailList);

            //开始封装要货计划请求
            System.out.println("erp请求为：" + JSON.parseObject(JSON.toJSONString(erpGoodsPlanReq)));

            Map erpContractData = new HashMap<>();
            erpContractData.put("data", Arrays.asList(erpGoodsPlanReq));//请求体

            String accessToken = new KDTokenController().getKDAccessTokenTest();

            if (accessToken == null || accessToken.equals("")) {
                log.info("金蝶token为空或不存在");
                return;
            }
            String uuid = UUID.randomUUID().toString().replace("-", "");
            //开始封装请求头
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(MediaType.APPLICATION_JSON);
            headers.set("accessToken", accessToken);
            headers.set("Idempotency-Key", uuid);

            HttpEntity goodsPlanReqEntity = new HttpEntity(erpContractData, headers);

            System.out.println(JSON.parseObject(JSON.toJSONString(goodsPlanReqEntity)));

            //开始向金蝶发起请求
            String goodsPlanUrl = "http://192.168.31.233:8022/ierp/kapi/v2/f9w5/sm/f9w5_sm_purchaseplan/purchaseplanSave";

            JSONObject erpGoodsPlanRes = new JSONObject();

            try {
                erpGoodsPlanRes = restTemplate.postForObject(goodsPlanUrl, goodsPlanReqEntity, JSONObject.class);
            } catch (RestClientException e) {
                e.printStackTrace();
            }

            System.out.println("要货计划结果为：" + JSON.parseObject(JSON.toJSONString(erpGoodsPlanRes)));

            //开始封装回写程序
            Map backReqMap = new HashMap();
            backReqMap.put("_id", goodsPlanId);
            backReqMap.put("dataObjectApiName", "purchase_plan__c");

            //开始封装日志
            Map logMap = new HashMap<>();
            logMap.put("log_id", UUID.randomUUID().toString().replace("-", ""));
            logMap.put("log_type", "GP");
            logMap.put("syn_type", "0");
            logMap.put("data_name", goodsPlanName);
            logMap.put("data_id", goodsPlanId);
            logMap.put("mark", createTime);
            logMap.put("send_body", JSON.toJSONString(goodsPlanReqEntity));
            logMap.put("send_res", JSON.toJSONString(erpGoodsPlanRes));
            logMap.put("tableName", "send_log_goodsplan");

            //判断是否执行成功
            if ("0".equals(erpGoodsPlanRes.getString("errorCode"))) {
                //同步成功
                logMap.put("log_status", "0");
                logMap.put("res_body", "同步成功");
                backReqMap.put("erp_id__c", erpGoodsPlanRes.getJSONObject("data").getJSONArray("result").getJSONObject(0).getString("id"));
                backReqMap.put("sync_status__c", "option_sync_success__c");
                backReqMap.put("response_info__c", "同步成功");
            } else {
                //同步失败
                logMap.put("log_status", "1");
                logMap.put("res_body", "同步失败：" + erpGoodsPlanRes.getString("message"));
                backReqMap.put("sync_status__c", "option_sync_failure__c");
                backReqMap.put("response_info__c", erpGoodsPlanRes.getString("message"));
            }

            //开始回写
            JSONObject backReq = crmRequestUtil.updateCRM(backReqMap);

            System.out.println("回写请求为：" + JSON.parseObject(JSON.toJSONString(backReq)));

            JSONObject backRes = new JSONObject();

            try {
                backRes = restTemplate.postForObject(URLConstant.UPDATE_CRM_CUSTOMIZE, backReq, JSONObject.class);
            } catch (RestClientException e) {
                e.printStackTrace();
            }

            System.out.println("回写结果为：" + backRes);

            //开始记录日志
            JSONObject logResMap = new JSONObject();
            try {
                logResMap = restTemplate.postForObject("http://localhost:18088/Log/insert/log_data", logMap, JSONObject.class);
            } catch (RestClientException e) {
                e.printStackTrace();
            }

            System.out.println("日志插入结果为：" + logResMap);


        }

    }

    /**
     * 要货计划更新接口
     *
     * @return void
     * @Author weiloong_zhang
     */
    @PostMapping("/update/goodsPlan")
    public void updateGoodsPlan() {
        CrmRequestUtil crmRequestUtil = new CrmRequestUtil();
        log.info("开始同步更新要货计划");

        //要货计划请求
        JSONObject getGoodsPlan = crmRequestUtil.getCRMList(Arrays.asList(
                new Filter("EQ", "name", Arrays.asList("YHJH20250508-0002")),
                new Filter("EQ", "life_status", Arrays.asList("normal"))
        ), "purchase_plan__c");

        JSONObject goodsPlanRes = new JSONObject();

        //开始发起请求
        try {
            goodsPlanRes = restTemplate.postForObject(URLConstant.GET_CUSTOMIZE_LIST_URL, getGoodsPlan, JSONObject.class);
        } catch (RestClientException e) {
            e.printStackTrace();
        }

        //判断是否请求成功
        if (!"success".equals(goodsPlanRes.getString("errorDescription")) || goodsPlanRes.getJSONObject("data").getJSONArray("dataList").isEmpty()) {
            log.info("请求失败或数据不存在");
            return;
        }

        //请求成功
        for (Object goodsPlanObj : goodsPlanRes.getJSONObject("data").getJSONArray("dataList")) {
            //将Obj转为JSONObject
            JSONObject goodsPlanData = JSON.parseObject(JSON.toJSONString(goodsPlanObj));

            System.out.println("当前正在处理的要货计划数据为：" + goodsPlanData);

            //获取当前单据的唯一id
            String goodsPlanId = goodsPlanData.getString("_id");

            //获取当前单据名称
            String goodsPlanName = goodsPlanData.getString("name");

            //获取当前单据的最后修改时间
            String lastModifyTime = goodsPlanData.getString("last_modified_time");

            //开始查询该单据是否已经集成过
            JSONObject logRes = new JSONObject();

            try {
                logRes = restTemplate.getForObject("http://localhost:18088/Log/query/log_data?table=send_log_goodsplan&log_type=GPUP&dataId=" + goodsPlanId + "&mark=" + lastModifyTime, JSONObject.class);
            } catch (RestClientException e) {
                e.printStackTrace();
            }

            if (!logRes.getJSONArray("data").isEmpty()) {
                log.info("当前单据已经更新过了");
                continue;
            }

            log.info("当前单据没有集成过");

            //开始封装传输至ERP的请求
            log.info("开始封装请求");

            //获取ERPID
            String erpId = goodsPlanData.getString("erp_id__c") != null ? goodsPlanData.getString("erp_id__c") : "";

            if ("".equals(erpId)) {
                log.info("当前单据erp_id__c为空，不再向下执行");
                continue;
            }

            log.info("存在ERPID，继续向下执行！");

            Map erpGoodsPlanReq = new HashMap<>();
            erpGoodsPlanReq.put("id", erpId);
            erpGoodsPlanReq.put("f9w5_crmid", goodsPlanId);
            erpGoodsPlanReq.put("billno", goodsPlanName);//单据编号
            erpGoodsPlanReq.put("billtype_number", "f9w5_sm_purchaseplan_BT");//单据类型
            erpGoodsPlanReq.put("org_number", "zyierp");//组织编码

            //开始获取需求日期
            long biztime = goodsPlanData.getLong("requirement_date__c");

            Date date = new Date(biztime);
            erpGoodsPlanReq.put("biztime", date);//需求日期

            //部门编码
            String deptCode = "";

            List deptIdList = goodsPlanData.getJSONArray("data_own_department");

            //开始查询部门编码
            JSONObject deptReq = crmRequestUtil.getCRMList(Arrays.asList(
                    new Filter("EQ", "_id", deptIdList)
            ), "DepartmentObj");

            JSONObject deptRes = new JSONObject();

            try {
                deptRes = restTemplate.postForObject(URLConstant.GET_CRM_LIST_URL, deptReq, JSONObject.class);
            } catch (RestClientException e) {
                e.printStackTrace();
            }

            if ("success".equals(deptRes.getString("errorDescription")) && !deptRes.getJSONObject("data").getJSONArray("dataList").isEmpty()) {
                JSONObject deptData = deptRes.getJSONObject("data").getJSONArray("dataList").getJSONObject(0);
                deptCode = deptData.getString("field_Vj6sf__c") != null ? deptData.getString("field_Vj6sf__c") : "";
            }

            erpGoodsPlanReq.put("dept", "zyierp");//todo 归属部门

            //获取负责人编码
            String ownerCode = "";

            List ownerIdList = goodsPlanData.getJSONArray("owner");

            JSONObject ownerReq = crmRequestUtil.getCRMList(Arrays.asList(
                    new Filter("EQ", "_id", ownerIdList)
            ), "PersonnelObj");

            JSONObject ownerRes = new JSONObject();

            try {
                ownerRes = restTemplate.postForObject(URLConstant.GET_CRM_LIST_URL, ownerReq, JSONObject.class);
            } catch (RestClientException e) {
                e.printStackTrace();
            }

            if ("success".equals(ownerRes.getString("errorDescription")) && !ownerRes.getJSONObject("data").getJSONArray("dataList").isEmpty()) {
                JSONObject ownerData = ownerRes.getJSONObject("data").getJSONArray("dataList").getJSONObject(0);
                ownerCode = ownerData.getString("field_T1xid__c") != null ? ownerData.getString("field_T1xid__c") : "";
            }

            erpGoodsPlanReq.put("operator", "ID-000011");//todo 负责人

            //根据要货计划查询要货计划明细
            JSONObject getGoodsPlanDetail = crmRequestUtil.getCRMList(Arrays.asList(
                    new Filter("EQ", "purchase_plan__c", Arrays.asList(goodsPlanId))
            ), "purchase_plan_details__c");

            JSONObject goodsPlanDetailRes = new JSONObject();

            try {
                goodsPlanDetailRes = restTemplate.postForObject(URLConstant.GET_CUSTOMIZE_LIST_URL, getGoodsPlanDetail, JSONObject.class);
            } catch (RestClientException e) {
                e.printStackTrace();
            }

            System.out.println("要货计划明细查询结果为：" + goodsPlanDetailRes);

            if (!"success".equals(goodsPlanDetailRes.getString("errorDescription")) || goodsPlanDetailRes.getJSONObject("data").getJSONArray("dataList").isEmpty()) {
                log.info("要货计划明细查询失败不存在当前要货计划明细");
                continue;
            }

            log.info("要货计划明细存在，继续向下执行");

            //明细列表
            List goodsPlanDetailList = new ArrayList<>();

            for (Object goodsPlanDetailObj : goodsPlanDetailRes.getJSONObject("data").getJSONArray("dataList")) {
                JSONObject goodsPlanDetailData = JSON.parseObject(JSON.toJSONString(goodsPlanDetailObj));

                System.out.println("当前正在处理的要货计划明细数据为：" + goodsPlanDetailData);

                //开始封装要货计划明细数据
                Map erpGoodsPlanDetailReq = new HashMap<>();
                erpGoodsPlanDetailReq.put("f9w5_crmentryid", goodsPlanDetailData.getString("_id"));//crm明细id
                erpGoodsPlanDetailReq.put("material_number", goodsPlanDetailData.getString("material_id__c__r"));//物料编号
                erpGoodsPlanDetailReq.put("entryorg_number", "zyierp");//组织编码
                erpGoodsPlanDetailReq.put("qty", goodsPlanDetailData.getBigDecimal("quantity__c") != null ? goodsPlanDetailData.getBigDecimal("quantity__c") : 0);//数量
                //erpGoodsPlanDetailReq.put("lot_number", goodsPlanDetailData.getString("batch_number__c") != null ? goodsPlanDetailData.getString("batch_number__c") : "");//批号
                erpGoodsPlanDetailReq.put("entrycomment", goodsPlanDetailData.getString("remark__c") != null ? goodsPlanDetailData.getString("remark__c") : "");//备注
                erpGoodsPlanDetailReq.put("f9w5_contractno", goodsPlanDetailData.getString("sales_contract_number__c__r") != null ? goodsPlanDetailData.getString("sales_contract_number__c__r") : "");//销售合同编号

                //开始查询项目号
                String projectId = goodsPlanDetailData.getString("project_name__c") != null ? goodsPlanDetailData.getString("project_name__c") : "";

                //开始查询项目号
                JSONObject getProjectReq = crmRequestUtil.getCRMList(Arrays.asList(
                        new Filter("EQ", "_id", Arrays.asList(projectId))
                ), "NewOpportunityObj");

                JSONObject projectRes = new JSONObject();

                try {
                    projectRes = restTemplate.postForObject(URLConstant.GET_CUSTOMIZE_LIST_URL, getProjectReq, JSONObject.class);
                } catch (RestClientException e) {
                    e.printStackTrace();
                }

                String projectCode = "";

                if ("success".equals(projectRes.getString("errorDescription")) && !projectRes.getJSONObject("data").getJSONArray("dataList").isEmpty()) {
                    JSONObject projectData = projectRes.getJSONObject("data").getJSONArray("dataList").getJSONObject(0);
                    projectCode = projectData.getString("opportunity_id__c") != null ? projectData.getString("opportunity_id__c") : "";
                }

                erpGoodsPlanDetailReq.put("f9w5_project_number", projectCode);//项目号

                goodsPlanDetailList.add(erpGoodsPlanDetailReq);

            }

            erpGoodsPlanReq.put("billentry", goodsPlanDetailList);

            //开始封装要货计划请求
            System.out.println("erp请求为：" + JSON.parseObject(JSON.toJSONString(erpGoodsPlanReq)));

            Map erpContractData = new HashMap<>();
            erpContractData.put("data", Arrays.asList(erpGoodsPlanReq));//请求体

            String accessToken = new KDTokenController().getKDAccessTokenTest();

            if (accessToken == null || accessToken.equals("")) {
                log.info("金蝶token为空或不存在");
                return;
            }
            String uuid = UUID.randomUUID().toString().replace("-", "");
            //开始封装请求头
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(MediaType.APPLICATION_JSON);
            headers.set("accessToken", accessToken);
            headers.set("Idempotency-Key", uuid);

            HttpEntity goodsPlanReqEntity = new HttpEntity(erpContractData, headers);

            System.out.println(JSON.parseObject(JSON.toJSONString(goodsPlanReqEntity)));

            //开始向金蝶发起请求
            String goodsPlanUrl = "http://192.168.31.233:8022/ierp/kapi/v2/f9w5/sm/f9w5_sm_purchaseplan/purchaseplanSave";

            JSONObject erpGoodsPlanRes = new JSONObject();

            try {
                erpGoodsPlanRes = restTemplate.postForObject(goodsPlanUrl, goodsPlanReqEntity, JSONObject.class);
            } catch (RestClientException e) {
                e.printStackTrace();
            }

            System.out.println("要货计划结果为：" + JSON.parseObject(JSON.toJSONString(erpGoodsPlanRes)));

            //开始封装回写程序
            Map backReqMap = new HashMap();
            backReqMap.put("_id", goodsPlanId);
            backReqMap.put("dataObjectApiName", "purchase_plan__c");

            //开始封装日志
            Map logMap = new HashMap<>();
            logMap.put("log_id", UUID.randomUUID().toString().replace("-", ""));
            logMap.put("log_type", "GPUP");
            logMap.put("syn_type", "1");
            logMap.put("data_name", goodsPlanName);
            logMap.put("data_id", goodsPlanId);
            logMap.put("mark", lastModifyTime);
            logMap.put("send_body", JSON.toJSONString(goodsPlanReqEntity));
            logMap.put("send_res", JSON.toJSONString(erpGoodsPlanRes));
            logMap.put("tableName", "send_log_goodsplan");

            //判断是否执行成功
            if ("0".equals(erpGoodsPlanRes.getString("errorCode"))) {
                //同步成功
                logMap.put("log_status", "0");
                logMap.put("res_body", "同步成功");
                backReqMap.put("erp_id__c", erpGoodsPlanRes.getJSONObject("data").getJSONArray("result").getJSONObject(0).getString("id"));
                backReqMap.put("sync_status__c", "option_sync_success__c");
                backReqMap.put("response_info__c", "同步成功");
            } else {
                //同步失败
                logMap.put("log_status", "1");
                logMap.put("res_body", "同步失败：" + erpGoodsPlanRes.getString("message"));
                backReqMap.put("sync_status__c", "option_sync_failure__c");
                backReqMap.put("response_info__c", erpGoodsPlanRes.getString("message"));
            }

            //开始回写
            JSONObject backReq = crmRequestUtil.updateCRM(backReqMap);

            System.out.println("回写请求为：" + JSON.parseObject(JSON.toJSONString(backReq)));

            JSONObject backRes = new JSONObject();

            try {
                backRes = restTemplate.postForObject(URLConstant.UPDATE_CRM_CUSTOMIZE, backReq, JSONObject.class);
            } catch (RestClientException e) {
                e.printStackTrace();
            }

            System.out.println("回写结果为：" + backRes);

            //开始记录日志
            JSONObject logResMap = new JSONObject();
            try {
                logResMap = restTemplate.postForObject("http://localhost:18088/Log/insert/log_data", logMap, JSONObject.class);
            } catch (RestClientException e) {
                e.printStackTrace();
            }

            System.out.println("日志插入结果为：" + logResMap);
        }
    }
}
