package org.ssssssss.magicboot.utils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;

import lombok.extern.slf4j.Slf4j;
import org.springframework.core.io.ByteArrayResource;
import org.springframework.http.*;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;
import org.ssssssss.magicboot.constant.CertificateConstant;
import org.ssssssss.magicboot.constant.URLConstant;
import org.ssssssss.magicboot.entity.Filter;

import java.util.*;

/**
 * 本工具类用于封装向纷享销客发送的请求参数，后续会完善其他的参数方法
 *
 * @param
 * @return null
 * @Author weiloong_zhang
 */
@Slf4j
public class CrmRequestUtil {

    private RestTemplate restTemplate = new RestTemplate();

    //封装根据条件查询数据列表的请求
    public JSONObject getCRMList(List<Filter> filters, String dataObjectApiName) {
        String corpAccessToken = "";
        //判断咖啡因中是否有数据
        if (CaffeineCacheUtil.get("corpAccessToken") != null) {
            log.info("咖啡因中有缓存的token数据");
            corpAccessToken = CaffeineCacheUtil.get("corpAccessToken").toString();
        } else {
            log.info("咖啡因中不存在token缓存");
            //封装请求
            Map tokenParams = new HashMap<>();
            tokenParams.put("appId", CertificateConstant.APP_ID);
            tokenParams.put("appSecret", CertificateConstant.APP_SECRET);
            tokenParams.put("permanentCode", CertificateConstant.PERMANENT_CODE);

            JSONObject tokenResponse = restTemplate.postForObject(URLConstant.GET_CORPACCESSTOKEN_URL, tokenParams, JSONObject.class);

            //判断是否获取成功
            if (tokenResponse.containsKey("errorMessage") && tokenResponse.getString("errorMessage").equals("success")) {
                corpAccessToken = tokenResponse.getString("corpAccessToken");
                CaffeineCacheUtil.put("corpAccessToken", corpAccessToken);
            } else {
                log.info("获取纷享销客token失败{}", tokenResponse);
            }
        }

        Map orderMap = new HashMap<>();
        orderMap.put("fieldName", "create_time");
        orderMap.put("isAsc", "false");

        Map searchQueryInfoMap = new HashMap<>();
        searchQueryInfoMap.put("offset", "0");
        searchQueryInfoMap.put("limit", "100");
        searchQueryInfoMap.put("orders", Arrays.asList(orderMap));
        searchQueryInfoMap.put("filters", filters);

        Map data = new HashMap<>();
        data.put("search_query_info", searchQueryInfoMap);
        data.put("dataObjectApiName", dataObjectApiName);

        Map request = new HashMap<>();
        request.put("corpAccessToken", corpAccessToken);
        request.put("currentOpenUserId", CertificateConstant.CURRENT_OPEN_USERID);
        request.put("corpId", CertificateConstant.CORP_ID);
        request.put("data", data);

        JSONObject requestJSON = new JSONObject(request);
        return requestJSON;
    }

    //封装更新单据的请求参数
    public JSONObject updateCRM(Map data) {
        String corpAccessToken = "";
        //判断咖啡因中是否有数据
        if (CaffeineCacheUtil.get("corpAccessToken") != null) {
            log.info("咖啡因中有缓存的token数据");
            corpAccessToken = CaffeineCacheUtil.get("corpAccessToken").toString();
        } else {
            log.info("咖啡因中不存在token缓存");
            //封装请求
            Map tokenParams = new HashMap<>();
            tokenParams.put("appId", CertificateConstant.APP_ID);
            tokenParams.put("appSecret", CertificateConstant.APP_SECRET);
            tokenParams.put("permanentCode", CertificateConstant.PERMANENT_CODE);

            JSONObject tokenResponse = restTemplate.postForObject(URLConstant.GET_CORPACCESSTOKEN_URL, tokenParams, JSONObject.class);

            //判断是否获取成功
            if (tokenResponse.containsKey("errorMessage") && tokenResponse.getString("errorMessage").equals("success")) {
                corpAccessToken = tokenResponse.getString("corpAccessToken");
            } else {
                log.info("获取纷享销客token失败{}", tokenResponse);
            }
        }

        //封装object_data
        Map objectData = new HashMap<>();
        objectData.put("object_data", data);

//        Map rootData = new HashMap<>();
//        rootData.put("data", objectData);

        Map request = new HashMap<>();
        request.put("corpAccessToken", corpAccessToken);
        request.put("currentOpenUserId", CertificateConstant.CURRENT_OPEN_USERID);
        request.put("corpId", CertificateConstant.CORP_ID);
        request.put("data", objectData);

        JSONObject requestJSON = new JSONObject(request);
        return requestJSON;
    }

    //封装新建单据的请求参数
    public JSONObject createCRM(Map map) {
        String corpAccessToken = "";
        //判断咖啡因中是否有数据
        if (CaffeineCacheUtil.get("corpAccessToken") != null) {
            log.info("咖啡因中有缓存的token数据");
            corpAccessToken = CaffeineCacheUtil.get("corpAccessToken").toString();
        } else {
            log.info("咖啡因中不存在token缓存");
            //封装请求
            Map tokenParams = new HashMap<>();
            tokenParams.put("appId", CertificateConstant.APP_ID);
            tokenParams.put("appSecret", CertificateConstant.APP_SECRET);
            tokenParams.put("permanentCode", CertificateConstant.PERMANENT_CODE);

            JSONObject tokenResponse = restTemplate.postForObject(URLConstant.GET_CORPACCESSTOKEN_URL, tokenParams, JSONObject.class);

            //判断是否获取成功
            if (tokenResponse.containsKey("errorMessage") && tokenResponse.getString("errorMessage").equals("success")) {
                corpAccessToken = tokenResponse.getString("corpAccessToken");
            } else {
                log.info("获取纷享销客token失败{}", tokenResponse);
            }
        }
        //封装object_data
        Map objectData = new HashMap<>();
        objectData.put("object_data", map);

        //Map rootData = new HashMap<>();
        //rootData.put("data", objectData);

        Map request = new HashMap<>();
        request.put("corpAccessToken", corpAccessToken);
        request.put("currentOpenUserId", CertificateConstant.CURRENT_OPEN_USERID);
        request.put("corpId", CertificateConstant.CORP_ID);
        request.put("data", objectData);

        JSONObject requestJSON = new JSONObject(request);
        return requestJSON;
    }

    /**
     * 生成crm请求数据
     *
     * @param crmId
     * @param dataObjectApiName:
     * @return com.alibaba.fastjson.JSONObject
     * @Author weiloong_zhang
     */
    public JSONObject unlockCRM(String crmId, String dataObjectApiName) {
        String corpAccessToken = "";
        //判断咖啡因中是否有数据
        if (CaffeineCacheUtil.get("corpAccessToken") != null) {
            log.info("咖啡因中有缓存的token数据");
            corpAccessToken = CaffeineCacheUtil.get("corpAccessToken").toString();
        } else {
            log.info("咖啡因中不存在token缓存");
            //封装请求
            Map tokenParams = new HashMap<>();
            tokenParams.put("appId", CertificateConstant.APP_ID);
            tokenParams.put("appSecret", CertificateConstant.APP_SECRET);
            tokenParams.put("permanentCode", CertificateConstant.PERMANENT_CODE);

            JSONObject tokenResponse = restTemplate.postForObject(URLConstant.GET_CORPACCESSTOKEN_URL, tokenParams, JSONObject.class);

            //判断是否获取成功
            if (tokenResponse.containsKey("errorMessage") && tokenResponse.getString("errorMessage").equals("success")) {
                corpAccessToken = tokenResponse.getString("corpAccessToken");
            } else {
                log.info("获取纷享销客token失败{}", tokenResponse);
            }
        }

        Map data = new HashMap<>();
        data.put("dataIds", Arrays.asList(crmId));
        data.put("dataObjectApiName", dataObjectApiName);
        Map request = new HashMap<>();
        request.put("corpAccessToken", corpAccessToken);
        request.put("currentOpenUserId", CertificateConstant.CURRENT_OPEN_USERID);
        request.put("corpId", CertificateConstant.CORP_ID);
        request.put("data", data);

        JSONObject requestJSON = new JSONObject(request);
        return requestJSON;
    }

    public JSONObject crmWithDetail(Map map, Map details) {
        String corpAccessToken = "";
        //判断咖啡因中是否有数据
        if (CaffeineCacheUtil.get("corpAccessToken") != null) {
            log.info("咖啡因中有缓存的token数据");
            corpAccessToken = CaffeineCacheUtil.get("corpAccessToken").toString();
        } else {
            log.info("咖啡因中不存在token缓存");
            //封装请求
            Map tokenParams = new HashMap<>();
            tokenParams.put("appId", CertificateConstant.APP_ID);
            tokenParams.put("appSecret", CertificateConstant.APP_SECRET);
            tokenParams.put("permanentCode", CertificateConstant.PERMANENT_CODE);

            JSONObject tokenResponse = restTemplate.postForObject(URLConstant.GET_CORPACCESSTOKEN_URL, tokenParams, JSONObject.class);

            //判断是否获取成功
            if (tokenResponse.containsKey("errorMessage") && tokenResponse.getString("errorMessage").equals("success")) {
                corpAccessToken = tokenResponse.getString("corpAccessToken");
            } else {
                log.info("获取纷享销客token失败{}", tokenResponse);
            }
        }
        //封装object_data
        Map objectData = new HashMap<>();
        objectData.put("object_data", map);
        objectData.put("details", details);

        Map request = new HashMap<>();
        request.put("corpAccessToken", corpAccessToken);
        request.put("currentOpenUserId", CertificateConstant.CURRENT_OPEN_USERID);
        request.put("corpId", CertificateConstant.CORP_ID);
        request.put("data", objectData);

        JSONObject requestJSON = new JSONObject(request);
        return requestJSON;
    }

    public JSONObject uploadFile(String dataId, String dataObjectApiName, String apiName, Map file) {
        String corpAccessToken = "";
        //判断咖啡因中是否有数据
        if (CaffeineCacheUtil.get("corpAccessToken") != null) {
            log.info("咖啡因中有缓存的token数据");
            corpAccessToken = CaffeineCacheUtil.get("corpAccessToken").toString();
        } else {
            log.info("咖啡因中不存在token缓存");
            //封装请求
            Map tokenParams = new HashMap<>();
            tokenParams.put("appId", CertificateConstant.APP_ID);
            tokenParams.put("appSecret", CertificateConstant.APP_SECRET);
            tokenParams.put("permanentCode", CertificateConstant.PERMANENT_CODE);

            JSONObject tokenResponse = restTemplate.postForObject(URLConstant.GET_CORPACCESSTOKEN_URL, tokenParams, JSONObject.class);

            //判断是否获取成功
            if (tokenResponse.containsKey("errorMessage") && tokenResponse.getString("errorMessage").equals("success")) {
                corpAccessToken = tokenResponse.getString("corpAccessToken");
            } else {
                log.info("获取纷享销客token失败{}", tokenResponse);
            }
        }

        Map fileMap = new HashMap<>();
        fileMap.put("_id", dataId);
        fileMap.put(apiName, Arrays.asList(file));
        fileMap.put("dataObjectApiName", dataObjectApiName);

        //封装object_data
        Map objectData = new HashMap<>();
        objectData.put("object_data", fileMap);

        Map request = new HashMap<>();
        request.put("corpAccessToken", corpAccessToken);
        request.put("currentOpenUserId", CertificateConstant.CURRENT_OPEN_USERID);
        request.put("corpId", CertificateConstant.CORP_ID);
        request.put("data", objectData);

        JSONObject requestJSON = new JSONObject(request);
        return requestJSON;

    }

    public String getCRMToken() {
        String corpAccessToken = "";
        //判断咖啡因中是否有数据
        if (CaffeineCacheUtil.get("corpAccessToken") != null) {
            log.info("咖啡因中有缓存的token数据");
            corpAccessToken = CaffeineCacheUtil.get("corpAccessToken").toString();
        } else {
            log.info("咖啡因中不存在token缓存");
            //封装请求
            Map tokenParams = new HashMap<>();
            tokenParams.put("appId", CertificateConstant.APP_ID);
            tokenParams.put("appSecret", CertificateConstant.APP_SECRET);
            tokenParams.put("permanentCode", CertificateConstant.PERMANENT_CODE);

            JSONObject tokenResponse = restTemplate.postForObject(URLConstant.GET_CORPACCESSTOKEN_URL, tokenParams, JSONObject.class);

            //判断是否获取成功
            if (tokenResponse.containsKey("errorMessage") && tokenResponse.getString("errorMessage").equals("success")) {
                corpAccessToken = tokenResponse.getString("corpAccessToken");
            } else {
                log.info("获取纷享销客token失败{}", tokenResponse);
            }
        }

        return corpAccessToken;
    }

    /**
     * 纷享销客文件转码base64
     *
     * @param fileList
     * @return java.lang.String[]
     * @Author weiloong_zhang
     */
    public MultiValueMap<String, Object> proxyUploadFileToTarget(List<?> fileList, String crmId) {
        MultiValueMap<String, Object> body = new LinkedMultiValueMap<>();
        body.add("crmId", crmId);
        body.add("fieldName", "f9w5_feedbackinfoattach");

        if (fileList == null || fileList.isEmpty()) {
            log.info("文件列表为空");
            return body;  // 返回空map而不是null
        }

        for (Object fileObj : fileList) {
            JSONObject fileData = JSON.parseObject(JSON.toJSONString(fileObj));

            Map<String, Object> fileReq = new HashMap<>();
            fileReq.put("corpAccessToken", getCRMToken());
            fileReq.put("corpId", CertificateConstant.CORP_ID);
            fileReq.put("mediaTypeDesc", "DOCUMENT");
            fileReq.put("mediaId", fileData.getString("path"));
            fileReq.put("igonreMediaIdConvert", false);

            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(MediaType.APPLICATION_JSON);
            HttpEntity<Map<String, Object>> fileReqEntity = new HttpEntity<>(fileReq, headers);

            try {
                ResponseEntity<byte[]> fileResp = restTemplate.exchange(
                        "https://open.fxiaoke.com/media/download",
                        HttpMethod.POST,
                        fileReqEntity,
                        byte[].class);

                if (fileResp.getStatusCode() == HttpStatus.OK && fileResp.getBody() != null) {
                    byte[] fileBytes = fileResp.getBody();
                    String fileName = fileData.getString("filename");
                    String contentType = fileData.getString("ext");

                    ByteArrayResource byteArrayResource = new ByteArrayResource(fileBytes) {
                        @Override
                        public String getFilename() {
                            return fileName != null ? fileName : UUID.randomUUID() + ".dat";
                        }
                    };

                    HttpHeaders fileHeaders = new HttpHeaders();
                    fileHeaders.setContentDispositionFormData("files", byteArrayResource.getFilename());

                    try {
                        fileHeaders.setContentType(MediaType.parseMediaType(
                                contentType != null ? contentType : MediaType.APPLICATION_OCTET_STREAM_VALUE));
                    } catch (Exception e) {
                        fileHeaders.setContentType(MediaType.APPLICATION_OCTET_STREAM);
                    }

                    HttpEntity<ByteArrayResource> fileEntity = new HttpEntity<>(byteArrayResource, fileHeaders);
                    body.add("files", fileEntity);
                } else {
                    log.error("文件下载失败，mediaId: {}", fileData.getString("path"));
                }
            } catch (Exception e) {
                log.error("下载文件出错", e);
            }
        }

        return body;
    }



}
